#ifndef GENERATOR_SINC_HH
#define GENERATOR_SINC_HH

#include "DataSource.hh"

namespace generator {

  /**  Sinc is an analytical function data source that generates a sinc
    *  of arbitrary frequency and amplitude. The generated waveform is: 
    *  \begin{center}
    *  \[ x(t)  = A  \sin(\omega(t-t_0)) / \omega(t-t_0)} \]
    *
    *  $ \omega = 2 \pi F $ 
    *  \end{center}
    *
    *  The wave-form parameters that may be specified by the generation
    *  control function are:
    *
    *  \begin{tabular}{lcl}
    *  Parameter & Default & Meaning \\
    *  #A#     & - & Amplitude at #t=to# \\
    *  #F#     & - & Sinc frequency (in Hz) \\
    *  #Width# & 6 & Half-width of the waveform to be generated (in 2/F units) \\
    *  \end{tabular} 
    *
    *  At the default width value of 6.0, the function is zero and the next
    *  peak would be #~0.05 A#.
    *  @memo Sinc function generator.
    *  @version 1.0; last modified June 15, 2004
    *  @author  John Zweizig
    */
  class Sinc : public DataSource {
  public:
      /**  Default constructor
        */
      Sinc(void);

      /**  Construct a SinGauss data source and initialize the waveform 
        *  parameters.
	*  @param Amp  the amplitude at #t=to#.
	*  @param Freq the sinusoid frequency (in Hz).
	*  @param wid  the half width of the generated data in units of 2/F.
        */
      Sinc(double Amp, double Freq, double wid=6.0);

      /** Destroy a SinGauss data source.
        */
      ~Sinc(void);

      /** Create an identical copy of a SinGauss object.
        */
      Sinc* clone(void) const;

      /**  Return a constant string containing the data source type, "SinGauss"
        */
      const char* getDataType(void) const;

      /**  Return the duration of the data to be generated. For SinGauss this
        *  is evaluated on an event by event basis and is #2*Width*Q/F#.
        */
      Interval getDuration(void) const;

  private:
      /**  Calculate the event data series.
        */
      void getSeries(const Time& t0, Interval dT, int N, float* data);
  };

  //====================================  Inline methods.
  inline const char*
  Sinc::getDataType(void) const {
      return "SinGauss";
  }

} // namespace generator

#endif  // GENERATOR_SINGAUSS_HH
