#include <detect.h>

#define NEG 1
#define POS 0

/*----------------------------------------------------------------------*
 *	Function: BOOL P_one() - Form the P-T series			*
 *		The algorithm for finding the peaks and troughs was	*
 *		  adapted from one by L.G. Holcomb.			*
 *----------------------------------------------------------------------*
 *	Arguments:	in_data, the seismic data sample		*
 *----------------------------------------------------------------------*
 *	Returns:	TRUE - Peak or trough detected			*
 *			FALSE - No Peak or trough detected		*
 *----------------------------------------------------------------------*
 *	Fatal Errors:	This routine does not have fatal exits		*
 *----------------------------------------------------------------------*
 *	This routine determines the signed amplitudes (and associated	*
 *	 times) of the filtered data. (By amplitudes, we mean the 	*
 *	 difference in value of the consecutive local maximums and	*
 *	 minimums, hereafter referred to as the peak-to-trough ampli-	*
 *	 tudes, or P-T value.)  Each peak or trough is determined by	*
 *	 comparing slopes between  the input samples.  When a peak	*
 *	 or trough is found, the record and time (ie, sample number)	*
 *	 where it occurred is documented.  In P_one, "y" denotes an	*
 *	 amplitude coordinate and "x" denotes a time coordinate.	*
 *									*
 *	In addition to calculating the peak-to-trough amplitudes,	*
 *	 P_one also compares each of these amplitudes to a threshold	*
 *	 thx.  The maximum of 20 successive values less than thx is 	*
 *	 fed to P_two.  P_two uses these maximums to calculate the 	*
 *	 statistical dispersion of the background noise.  The thresh-	*
 *	 hold is used to inhibit anomalously large values (such as	*
 *	 spikes) from contributing to the estimate of the normal	*
 *	 background.							*
 *----------------------------------------------------------------------*/

BOOL P_one(in_data)
register    LONG in_data;

{
	register    BOOL cur_slope;
	LONG del_amp, ab_amp;
	LONG samSec;

	del_amp = in_data - con_ptr->last_y;

	con_ptr->sum_s_c++;

	if (del_amp < 0)
	    cur_slope = NEG;
	else
	    cur_slope = POS;

/*-------------Was the last sample a peak or trough?------------------------*/
						 /* Note 1 */
	if (con_ptr->prev_slope != cur_slope) {
			/* a maximum or minimum has been found */
	    con_ptr->s_amp = con_ptr->max_y - con_ptr->last_y;
	    con_ptr->tim_of_max = con_ptr->last_x;
	    con_ptr->rec_of_max = con_ptr->rec_last_x;

	    con_ptr->prev_slope = cur_slope;   /* slope between samples */
	    con_ptr->max_y = con_ptr->last_y;/* coordinate of max or min */

	    con_ptr->last_y = in_data;      /* current amplitude coordinate */
	    con_ptr->last_x = sam_no;    /* current time coordinate */

	    con_ptr->rec_last_x = con_ptr->cur_rec;

	    con_ptr->s_sum_sc = con_ptr->sum_s_c;/* save sum */
	    con_ptr->sum_s_c = 0;	 /* initialize sum of sample counts */

/*------Get max of one second's worth of  P-T values less than thx,----------*/

	    if ((ab_amp = absval (con_ptr->s_amp)) >= con_ptr->thx)
		return(TRUE);	       /* P-T >= thx */

	    if (ab_amp > con_ptr->maxamp)
		con_ptr->maxamp = ab_amp;

	    samSec = con_ptr->sam_sec;

	    if (samSec == 128) { 

	      if (++con_ptr->index == 20) {	/* increment index */

/*----------------------and send this maximum to P_two.-----------------------*/

		P_two(con_ptr->maxamp);

		/* reset for the next 20 */
		con_ptr->maxamp = 0L;
		con_ptr->index = 0;
	      }
	    } else if (samSec == 256){
	      if (++con_ptr->index == 32) {	/* increment index */
		P_two(con_ptr->maxamp);

		/* reset for the next 32 */
		con_ptr->maxamp = 0L;
		con_ptr->index = 0;
	      }
	    }

	    else if (samSec == 1024){    
	                             /* Half-second for 16K channels */
	      if (++con_ptr->index == 64) {	/* increment index */
		P_two(con_ptr->maxamp);

		/* reset for the next 64 */
		con_ptr->maxamp = 0L;
		con_ptr->index = 0;
	      }
	    } else {

	      printf("Error: Downsampled rate must be 128 or 256 or 1024");
	    }
	    return(TRUE);		       /* P-T < thx */

	}

	con_ptr->last_y = in_data;       /* current amplitude coordinate */
	con_ptr->last_x = sam_no;	       /* current time coordinate */
				/* Note 2 */
	con_ptr->rec_last_x = con_ptr->cur_rec;

	return(FALSE);		       /* peak or trough not found */

}

#ifdef JNMCOMMENT
/******************************************************************************

				NOTES

	[1]	If a change in slope occurs, the last sample was a peak 
		 or trough.  Calculate the peak-to-trough amplitude
		 and document the time at which this amplitude occurred.

	[2]	The peak or trough might occur in the last sample
		 of the previous record.  Keep track of in which record
		 the peak or trough occurrs.


********************************************************************************

			EXPLANATION OF VARIABLES

	ab_amp     -- the absolute value of the peak-to-trough difference.

	cur_slope  -- slope (positive or negative) between the last data
		       sample (amplitude coordinate) and the current one.

	del_amp    -- the current amplitude coordinate minus the previous one.

	in_data	   -- the current data sample.

	index	   -- a counter that is used to control calls to P_two.

	last_y     -- amplitude coordinate immediately preceeding the
		       current one.

	maxamp	   -- the absolute maximum of 20 successive P-T values < thx.

	max_y      -- maximum (or minimum) amplitude coordinate of the
		       last one-half cycle. (It is the amplitude coordinate 
		       of the last peak or trough.)

	prev_slope -- slope (positive or negative) between the two samples
		       (amplitude coordinates) immediately prior to the 
			current one.

	rec_last_x -- record in which the previous time coordinate occurred.

	rec_of_max -- the record in which the peak or trough occurred.

	s_amp      -- signed amplitude (signed peak-to-trough difference).

	s_sum_sc   -- the number of samples between the last two
		       P-T values.  (It is the number of samples between
		       the last peak and trough.)

	sam_no	   -- the current sample number.  Set in E_detect.

	sum_s_c    -- sum of sample counts in the interval from the last
		       peak or trough to the current sample.

	tim_of_max -- time coordinate of the last one-half cycle. (It is the
		       time coordinate of the last peak or trough, ie of the 
		       last P-T value.)

*******************************************************************************/
#endif
